from dataclasses import dataclass
from datetime import datetime

from psycopg import sql
from psycopg.rows import class_row
from psycopg_pool import AsyncConnectionPool


@dataclass
class Member:
    id: int
    email: str
    password_hash: str
    created: datetime
    email_verified: datetime | None


async def select_member_by_email(db: AsyncConnectionPool, email: str) -> Member | None:
    """
    Find exactly one email match in users list in PostgreSQL database
    """
    async with db.connection() as conn:
        async with conn.cursor(row_factory=class_row(Member)) as curr:
            query = sql.SQL(
                """
                    SELECT id, email, password_hash, created, email_verified
                        FROM members
                        WHERE LOWER(email) = LOWER(%s);
                    """
            )

            await curr.execute(query, (email,))

            member = await curr.fetchone()

            return None if member is None else member


async def select_member_by_id(db: AsyncConnectionPool, id: int) -> Member | None:
    """
    Find exactly one id match in users list in PostgreSQL database
    """
    async with db.connection() as conn:
        async with conn.cursor(row_factory=class_row(Member)) as curr:
            query = sql.SQL(
                """
                    SELECT id, email, password_hash, created, email_verified
                        FROM members
                        WHERE id = (%s);
                    """
            )

            await curr.execute(query, (id,))

            member = await curr.fetchone()

            return None if member is None else member


async def insert_member(
    db: AsyncConnectionPool, email: str, password_hash: str
) -> Member | None:
    """
    Insert a member into the database

    Return the inserted member object
    """
    async with db.connection() as conn:
        async with conn.cursor(row_factory=class_row(Member)) as curr:
            query = sql.SQL(
                """
                INSERT INTO members (email, password_hash)
                    VALUES (%s, %s)
                    RETURNING id, email, password_hash, created, email_verified
                """
            )

            await curr.execute(
                query,
                (
                    email,
                    password_hash,
                ),
            )

            member = await curr.fetchone()

            return None if member is None else member


async def update_member_password(
    db: AsyncConnectionPool, id: int, password_hash: str
) -> bool:
    async with db.connection() as conn:
        async with conn.cursor(row_factory=class_row(Member)) as curr:
            query = sql.SQL(
                """
                UPDATE members
                    SET password_hash = (%s)
                    WHERE id = (%s)
                """
            )

            await curr.execute(
                query,
                (
                    password_hash,
                    id,
                ),
            )

            return curr.rowcount > 0


async def update_member_email_verified(db: AsyncConnectionPool, id: int) -> bool:
    async with db.connection() as conn:
        async with conn.cursor(row_factory=class_row(Member)) as curr:
            query = sql.SQL(
                """
                UPDATE members SET email_verified = now() WHERE id = (%s)
                """
            )

            await curr.execute(
                query,
                (id,),
            )

            return curr.rowcount > 0
